"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IndyVdrIndyDidRegistrar = void 0;
const core_1 = require("@aries-framework/core");
const indy_vdr_shared_1 = require("@hyperledger/indy-vdr-shared");
const error_1 = require("../error");
const IndyVdrPoolService_1 = require("../pool/IndyVdrPoolService");
const didIndyUtil_1 = require("./didIndyUtil");
const didSovUtil_1 = require("./didSovUtil");
class IndyVdrIndyDidRegistrar {
    constructor() {
        this.supportedMethods = ['indy'];
    }
    async create(agentContext, options) {
        var _a, _b, _c, _d;
        const seed = (_a = options.secret) === null || _a === void 0 ? void 0 : _a.seed;
        const privateKey = (_b = options.secret) === null || _b === void 0 ? void 0 : _b.privateKey;
        const { alias, role, submitterDid, services, useEndpointAttrib } = options.options;
        let did = options.did;
        let namespaceIdentifier;
        let verificationKey;
        const allowOne = [privateKey, seed, did].filter((e) => e !== undefined);
        if (allowOne.length > 1) {
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {},
                didState: {
                    state: 'failed',
                    reason: `Only one of 'seed', 'privateKey' and 'did' must be provided`,
                },
            };
        }
        try {
            // Parse submitterDid and extract namespace based on the submitter did
            const { namespace: submitterNamespace, namespaceIdentifier: submitterNamespaceIdentifier } = (0, didIndyUtil_1.parseIndyDid)(submitterDid);
            const submitterSigningKey = await (0, didIndyUtil_1.verificationKeyForIndyDid)(agentContext, submitterDid);
            if (did) {
                if (!options.options.verkey) {
                    return {
                        didDocumentMetadata: {},
                        didRegistrationMetadata: {},
                        didState: {
                            state: 'failed',
                            reason: 'If a did is defined, a matching verkey must be provided',
                        },
                    };
                }
                const { namespace, namespaceIdentifier: _namespaceIdentifier } = (0, didIndyUtil_1.parseIndyDid)(did);
                namespaceIdentifier = _namespaceIdentifier;
                verificationKey = core_1.Key.fromPublicKeyBase58(options.options.verkey, core_1.KeyType.Ed25519);
                if (!(0, didIndyUtil_1.isSelfCertifiedIndyDid)(did, options.options.verkey)) {
                    return {
                        didDocumentMetadata: {},
                        didRegistrationMetadata: {},
                        didState: {
                            state: 'failed',
                            reason: `Initial verkey ${options.options.verkey} does not match did ${did}`,
                        },
                    };
                }
                if (submitterNamespace !== namespace) {
                    return {
                        didDocumentMetadata: {},
                        didRegistrationMetadata: {},
                        didState: {
                            state: 'failed',
                            reason: `The submitter did uses namespace ${submitterNamespace} and the did to register uses namespace ${namespace}. Namespaces must match.`,
                        },
                    };
                }
            }
            else {
                // Create a new key and calculate did according to the rules for indy did method
                verificationKey = await agentContext.wallet.createKey({ privateKey, seed, keyType: core_1.KeyType.Ed25519 });
                const buffer = core_1.Hasher.hash(verificationKey.publicKey, 'sha2-256');
                namespaceIdentifier = core_1.TypedArrayEncoder.toBase58(buffer.slice(0, 16));
                did = `did:indy:${submitterNamespace}:${namespaceIdentifier}`;
            }
            // Create base did document
            const didDocumentBuilder = (0, didIndyUtil_1.indyDidDocumentFromDid)(did, verificationKey.publicKeyBase58);
            let diddocContent;
            // Add services if object was passed
            if (services) {
                services.forEach((item) => {
                    const prependDidIfNotPresent = (id) => {
                        return id.startsWith('#') ? `${did}${id}` : id;
                    };
                    // Prepend the did to the service id if it is not already there
                    item.id = prependDidIfNotPresent(item.id);
                    // TODO: should we also prepend the did to routingKeys?
                    if (item instanceof core_1.DidCommV1Service) {
                        item.recipientKeys = item.recipientKeys.map(prependDidIfNotPresent);
                    }
                    didDocumentBuilder.addService(item);
                });
                const commTypes = [core_1.IndyAgentService.type, core_1.DidCommV1Service.type, core_1.DidCommV2Service.type];
                const serviceTypes = new Set(services.map((item) => item.type));
                const keyAgreementId = `${did}#key-agreement-1`;
                // If there is at least a communication service, add the key agreement key
                if (commTypes.some((type) => serviceTypes.has(type))) {
                    didDocumentBuilder
                        .addContext('https://w3id.org/security/suites/x25519-2019/v1')
                        .addVerificationMethod({
                        controller: did,
                        id: keyAgreementId,
                        publicKeyBase58: (0, didIndyUtil_1.createKeyAgreementKey)(verificationKey.publicKeyBase58),
                        type: 'X25519KeyAgreementKey2019',
                    })
                        .addKeyAgreement(keyAgreementId);
                }
                // If there is a DIDComm V2 service, add context
                if (serviceTypes.has(core_1.DidCommV2Service.type)) {
                    didDocumentBuilder.addContext('https://didcomm.org/messaging/contexts/v2');
                }
                if (!useEndpointAttrib) {
                    // create diddocContent parameter based on the diff between the base and the resulting DID Document
                    diddocContent = (0, didIndyUtil_1.didDocDiff)(didDocumentBuilder.build().toJSON(), (0, didIndyUtil_1.indyDidDocumentFromDid)(did, verificationKey.publicKeyBase58).build().toJSON());
                }
            }
            // Build did document
            const didDocument = didDocumentBuilder.build();
            const pool = agentContext.dependencyManager.resolve(IndyVdrPoolService_1.IndyVdrPoolService).getPoolForNamespace(submitterNamespace);
            // If there are services and we are using legacy indy endpoint attrib, make sure they are suitable before registering the DID
            if (services && useEndpointAttrib) {
                const endpoints = (0, didSovUtil_1.endpointsAttribFromServices)(services);
                await this.registerPublicDid(agentContext, pool, submitterNamespaceIdentifier, submitterSigningKey, namespaceIdentifier, verificationKey, alias, role);
                await this.setEndpointsForDid(agentContext, pool, namespaceIdentifier, verificationKey, endpoints);
            }
            else {
                await this.registerPublicDid(agentContext, pool, submitterNamespaceIdentifier, submitterSigningKey, namespaceIdentifier, verificationKey, alias, role, diddocContent);
            }
            // Save the did so we know we created it and can issue with it
            const didRecord = new core_1.DidRecord({
                did,
                role: core_1.DidDocumentRole.Created,
                tags: {
                    recipientKeyFingerprints: didDocument.recipientKeys.map((key) => key.fingerprint),
                    qualifiedIndyDid: did,
                },
            });
            const didRepository = agentContext.dependencyManager.resolve(core_1.DidRepository);
            await didRepository.save(agentContext, didRecord);
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {
                    didIndyNamespace: submitterNamespace,
                },
                didState: {
                    state: 'finished',
                    did,
                    didDocument,
                    secret: {
                        // FIXME: the uni-registrar creates the seed in the registrar method
                        // if it doesn't exist so the seed can always be returned. Currently
                        // we can only return it if the seed was passed in by the user. Once
                        // we have a secure method for generating seeds we should use the same
                        // approach
                        seed: (_c = options.secret) === null || _c === void 0 ? void 0 : _c.seed,
                        privateKey: (_d = options.secret) === null || _d === void 0 ? void 0 : _d.privateKey,
                    },
                },
            };
        }
        catch (error) {
            return {
                didDocumentMetadata: {},
                didRegistrationMetadata: {},
                didState: {
                    state: 'failed',
                    reason: `unknownError: ${error.message}`,
                },
            };
        }
    }
    async update() {
        return {
            didDocumentMetadata: {},
            didRegistrationMetadata: {},
            didState: {
                state: 'failed',
                reason: `notImplemented: updating did:indy not implemented yet`,
            },
        };
    }
    async deactivate() {
        return {
            didDocumentMetadata: {},
            didRegistrationMetadata: {},
            didState: {
                state: 'failed',
                reason: `notImplemented: deactivating did:indy not implemented yet`,
            },
        };
    }
    async registerPublicDid(agentContext, pool, unqualifiedSubmitterDid, submitterSigningKey, unqualifiedDid, signingKey, alias, role, diddocContent) {
        try {
            agentContext.config.logger.debug(`Register public did '${unqualifiedDid}' on ledger '${pool}'`);
            // FIXME: Add diddocContent when supported by indy-vdr
            if (diddocContent) {
                throw new error_1.IndyVdrError('diddocContent is not yet supported');
            }
            const request = new indy_vdr_shared_1.NymRequest({
                submitterDid: unqualifiedSubmitterDid,
                dest: unqualifiedDid,
                verkey: signingKey.publicKeyBase58,
                alias,
            });
            const response = await pool.submitWriteRequest(agentContext, request, submitterSigningKey);
            agentContext.config.logger.debug(`Registered public did '${unqualifiedDid}' on ledger '${pool.indyNamespace}'`, {
                response,
            });
            return;
        }
        catch (error) {
            agentContext.config.logger.error(`Error registering public did '${unqualifiedDid}' on ledger '${pool.indyNamespace}'`, {
                error,
                unqualifiedSubmitterDid,
                unqualifiedDid,
                signingKey,
                alias,
                role,
                pool: pool.indyNamespace,
            });
            throw error;
        }
    }
    async setEndpointsForDid(agentContext, pool, unqualifiedDid, signingKey, endpoints) {
        try {
            agentContext.config.logger.debug(`Set endpoints for did '${unqualifiedDid}' on ledger '${pool.indyNamespace}'`, endpoints);
            const request = new indy_vdr_shared_1.AttribRequest({
                submitterDid: unqualifiedDid,
                targetDid: unqualifiedDid,
                raw: JSON.stringify({ endpoint: endpoints }),
            });
            const response = await pool.submitWriteRequest(agentContext, request, signingKey);
            agentContext.config.logger.debug(`Successfully set endpoints for did '${unqualifiedDid}' on ledger '${pool.indyNamespace}'`, {
                response,
                endpoints,
            });
        }
        catch (error) {
            agentContext.config.logger.error(`Error setting endpoints for did '${unqualifiedDid}' on ledger '${pool.indyNamespace}'`, {
                error,
                unqualifiedDid,
                endpoints,
            });
            throw new error_1.IndyVdrError(error);
        }
    }
}
exports.IndyVdrIndyDidRegistrar = IndyVdrIndyDidRegistrar;
//# sourceMappingURL=IndyVdrIndyDidRegistrar.js.map